/*
 * Copyright (c) 2003, 2004, 2005 Israfil Consulting Services Corporation
 * Copyright (c) 2003, 2004, 2005 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: DynamicallyMutableObject.java 36 2006-02-17 01:59:38Z cgruber $
 */
package net.israfil.foundation.core;

import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * An abstract class which implements the DynamicallyMutableObject 
 * infrastructure.  It sets a field's value, by mutator method if such 
 * exists, or directly if none such exists.
 * 
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 *
 */
//@Copyright(years={"2003","2004","2005","2006"},owner="Israfil Consulting Services Corporation",license="BSD")
public abstract class DynamicallyMutableObject extends DynamicallyAccessibleObject implements DynamicallyMutable {
	private static Logger logger = Logger.getLogger(DynamicallyMutableObject.class.getName());
   

	public void set(String attributeName,Object value) {
		
		if (hasMutator(attributeName,value.getClass())) {
			this.perform(getMutatorSelector(attributeName,value.getClass()),new Object[]{value});
			return;
		} else if(hasMutator(attributeName,DynamicUtil.getPrimitiveTypeEquivalent(value.getClass()))) {
			this.perform(getMutatorSelector(attributeName,DynamicUtil.getPrimitiveTypeEquivalent(value.getClass())),new Object[]{value});
			return;
		} else {
			Field f;
			try {
				f = this.getClass().getField(attributeName);
				f.set(this,value);
			} catch (NoSuchFieldException e) {
				logger.log(Level.FINEST,"Object attempted to dynamically access a non-existant or unreadable field.",e);
			} catch (IllegalAccessException e) {
				logger.log(Level.FINEST,"Object attempted to dynamically access a inaccessible field.",e);
			}
		}
	}
	
	private static String mutatePrefix =  "set";
	private String getMutatorSelector(String attributeName,Class type) {
		if (attributeName == null || attributeName.equals("") || type == null) return null;
		String camelAttr = camel(attributeName);
		Iterator i = DynamicUtil.getAllParentTypes(type).iterator();
		while (i.hasNext()) {
			Class c = (Class)i.next();
			String selector = mutatePrefix+camelAttr+":"+c.getName();
			if (respondsTo(selector)) return selector;	
		}
		String selector = mutatePrefix+camelAttr+":"+type.getName();
		if (respondsTo(selector)) return selector;	
		return null;
	}	
 
	public boolean hasMutator(String attributeName,Class type) {
		return getMutatorSelector(attributeName,type) != null;
	}
}
